Attribute VB_Name = "DblMetaphone"
Option Explicit


' This code is released to the public domain.
' Jason Filley, 2002

' Double Metaphone is the creation of Lawrence Philips
' http://www.cuj.com/articles/2000/0006/0006d/0006d.htm?topic=articles
'
' The main portion is converted from C++ to VB, and there may
' be faster ways, but any future modifications will be easy to
' match.
'
' And it could be faster.  It could be slower.


'USAGE:  DoubleMetaphone(StringYouWantAnalyzed).
'                                               BaseWord = The Base String
'                                               Primary = Primary Metaphone value
'                                               Secondary = Secondary value (always exists if Primary does)
'                                               HasAlternate = True if Primary and Secondary are different
'
'To print the base string, the primary DM value, and the secondary value (only if different from the primary)
'then (more or less):
'
'LineIn = "Whatever"
'
'If DoubleMetaphone(LineIn).HasAlternate = False Then
'    LineOut = DoubleMetaphone(LineIn).BaseWord & vbTab & _
'    DoubleMetaphone(LineIn).Primary & vbCrLf
'
'Else
'    LineOut = DoubleMetaphone(LineIn).BaseWord & vbTab & _
'    DoubleMetaphone(LineIn).Primary & vbTab & _
'    DoubleMetaphone(LineIn).Secondary & vbCrLf
'
'End If
'
'Debug.Print LineOut

Public Type DMValue

    BaseWord As String
    Primary As String
    Secondary As String
    HasAlternate As Boolean
    
End Type

Private Function MyMid(ByVal InputString As String, ByVal Start As Long, Optional ByVal Length As Long) As String
' Mid throws fits if the start position is less than 1

If Start < 1 Then
    MyMid = vbNull
Else
    MyMid = Mid$(InputString, Start, Length)
End If

End Function

Private Function IsSlavoGermanic(ByVal InputString As String) As Boolean
' Should only check this once per string ?

  If ContainsAnyOf(InputString, "W", "K", "CZ", "WITZ") Then
    IsSlavoGermanic = True
  End If
  
End Function


Function ContainsAnyOf(ByVal BaseStringToSearch As String, ParamArray SubstringsToSearchFor()) As Boolean

Dim CurrentlySearching As Variant


For Each CurrentlySearching In SubstringsToSearchFor()
  If InStr(1, BaseStringToSearch, CurrentlySearching) Then
    ContainsAnyOf = True
  Exit For
  End If
Next CurrentlySearching

End Function


Public Function DoubleMetaphone(ByVal InputString As String) As DMValue

Dim iCurrentPosition As Integer
Dim CurrentChar As String
Dim Last As Integer
Dim tmpPrimary As String
Dim tmpSecondary As String


iCurrentPosition = 1
Last = Len(InputString)
tmpPrimary = vbNullString
tmpSecondary = vbNullString

' Might be faster arranging the cases as <vowels> and "TNSRHLDCPFMWBGVKQXJZ" ?
' ETAONISRHLDCUPFMWYBGVKQXJZ and TNSRHLDCPFMWBGVKQXJZ

DoubleMetaphone.BaseWord = InputString
InputString = UCase$(InputString)
InputString = InputString & "       "

' skip these when at start of word
If ContainsAnyOf(Left$(InputString, 2), "GN", "KN", "PN", "WR", "PS") Then
    iCurrentPosition = iCurrentPosition + 1
End If

' Initial 'X' is pronounced 'Z' e.g. 'Xavier'
If Left$(InputString, 1) = "X" Then
    tmpPrimary = tmpPrimary & "S"
    tmpSecondary = tmpSecondary & "S"
    iCurrentPosition = iCurrentPosition + 1
End If


Do Until iCurrentPosition > Last

GetOutOfSelect: ' Goto's are ugly, but it's easy
CurrentChar = Mid$(InputString, iCurrentPosition, 1)


Select Case CurrentChar


Case "A", "E", "I", "O", "U", "Y"

        If iCurrentPosition = 1 Then
            tmpPrimary = tmpPrimary & "A"
            tmpSecondary = tmpSecondary & "A"
        End If

        iCurrentPosition = iCurrentPosition + 1

Case "B"

        ' "-mb", e.g", "dumb", already skipped over...
        tmpPrimary = tmpPrimary & "P"
        tmpSecondary = tmpSecondary & "P"

        If Mid$(InputString, iCurrentPosition + 1, 1) = "B" Then
            iCurrentPosition = iCurrentPosition + 2
        Else
            iCurrentPosition = iCurrentPosition + 1
        End If

Case ""

        tmpPrimary = tmpPrimary & "S"
        tmpSecondary = tmpSecondary & "S"
        iCurrentPosition = iCurrentPosition + 1

Case "C"
    ' various germanic
    If iCurrentPosition > 2 _
    And (InStr(1, "AEIOUY", MyMid$(InputString, iCurrentPosition - 2, 1)) < 1) _
    And MyMid$(InputString, iCurrentPosition - 1, 3) = "ACH" _
    And ((Mid$(InputString, iCurrentPosition + 2, 1) <> "I") _
    And (Mid$(InputString, iCurrentPosition + 2, 1) <> "E" _
    Or (MyMid$(InputString, iCurrentPosition - 2, 6) = "BACHER" _
        Or MyMid$(InputString, iCurrentPosition - 2, 6) = "MACHER"))) Then
        
        tmpPrimary = tmpPrimary & "K"
        tmpSecondary = tmpSecondary & "K"
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
    End If

    ' special case 'caesar'
    If iCurrentPosition = 1 And Left$(InputString, 6) = "CAESAR" Then
        tmpPrimary = tmpPrimary & "S"
        tmpSecondary = tmpSecondary & "S"
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
    End If

    ' Italian "chianti"
    If Mid$(InputString, iCurrentPosition, 4) = "CHIA" Then
        tmpPrimary = tmpPrimary & "K"
        tmpSecondary = tmpSecondary & "K"
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
    End If

    If Mid$(InputString, iCurrentPosition, 2) = "CH" Then
    
        ' find 'michael'
        If iCurrentPosition > 1 And Mid$(InputString, iCurrentPosition, 4) = "CHAE" Then

            tmpPrimary = tmpPrimary & "K"
            tmpSecondary = tmpSecondary & "X"
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
        End If

        ' Greek roots e.g. "chemistry" and "chorus"
        If (iCurrentPosition = 1 _
        And ((Mid$(InputString, iCurrentPosition + 1, 5) = "HARAC" _
            Or Mid$(InputString, iCurrentPosition + 1, 5) = "HARIS") _
        Or ContainsAnyOf(Mid$(InputString, iCurrentPosition + 1, 3), "HOR", "HYM", "HIA", "HEM") _
        And Mid$(InputString, 1, 5) <> "CHORE")) Then
                
            tmpPrimary = tmpPrimary & "K"
            tmpSecondary = tmpSecondary & "K"
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
        End If

        ' Germanic, Greek, or otherwise "ch" for "kh" sound
        If (Left$(InputString, 4) = "VAN " Or Left$(InputString, 4) = "VON " _
            Or Left$(InputString, 3) = "SCH") _
        Or ContainsAnyOf(MyMid$(InputString, iCurrentPosition - 2, 6), "ORCHES", "ARCHIT", "ORCHID") _
        Or InStr(1, "TS", MyMid$(InputString, iCurrentPosition + 2, 1)) _
        Or (((InStr(1, "AOUE", MyMid$(InputString, iCurrentPosition - 1, 1)) > 0) Or iCurrentPosition = 1) _
        And InStr(1, "LRNMBHFVW ", Mid$(InputString, iCurrentPosition + 2, 1))) Then

            tmpPrimary = tmpPrimary & "K"
            tmpSecondary = tmpSecondary & "K"
            
                 
        Else
            If iCurrentPosition > 1 Then

                If Left$(InputString, 2) = "MC" Then
                    ' e.g., "McHugh"

                    tmpPrimary = tmpPrimary & "K"
                    tmpSecondary = tmpSecondary & "K"
                Else

                    tmpPrimary = tmpPrimary & "X"
                    tmpSecondary = tmpSecondary & "K"
                End If
            Else

                tmpPrimary = tmpPrimary & "X"
                tmpSecondary = tmpSecondary & "X"
            End If
        End If
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
        
    End If
    
    ' e.g, 'czerny'
    If Mid$(InputString, iCurrentPosition, 2) = "CZ" _
        And MyMid$(InputString, iCurrentPosition - 2) <> "CZ" Then
            
            tmpPrimary = tmpPrimary & "S"
            tmpSecondary = tmpSecondary & "X"
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
    End If

    ' e.g., 'focaccia'
    If Mid$(InputString, iCurrentPosition + 1, 3) = "CIA" Then

        tmpPrimary = tmpPrimary & "X"
        tmpSecondary = tmpSecondary & "X"
        iCurrentPosition = iCurrentPosition + 3
        GoTo GetOutOfSelect
    End If

    ' double 'C', but not if e.g. 'McClellan'
    If (Mid$(InputString, iCurrentPosition, 2) = "CC") _
    And Not (iCurrentPosition = 1 And Left$(InputString, 1) = "M") Then
    
        ' 'bellocchio' but not 'bacchus'
        If InStr(1, "IEH", Mid$(InputString, iCurrentPosition + 2, 1)) _
        And Mid$(InputString, iCurrentPosition + 2, 2) <> "HU" Then

            ' 'accident', 'accede' 'succeed'
            If iCurrentPosition = 2 And Left$(InputString, 1) = "A" _
            Or (MyMid$(InputString, iCurrentPosition - 1, 5) = "UCCEE" _
                Or MyMid$(InputString, iCurrentPosition - 1, 5) = "UCCES") Then

                tmpPrimary = tmpPrimary & "KS"
                tmpSecondary = tmpSecondary & "KS"
                ' 'bacci', 'bertucci', other italian
            Else

                tmpPrimary = tmpPrimary & "X"
                tmpSecondary = tmpSecondary & "X"
            End If
            
            iCurrentPosition = iCurrentPosition + 3
            GoTo GetOutOfSelect
            
        Else ' Pierce's rule

            tmpPrimary = tmpPrimary & "K"
            tmpSecondary = tmpSecondary & "K"
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
            
        End If
    End If

    If ContainsAnyOf(Mid$(InputString, iCurrentPosition, 2), "CK", "CG", "CQ") Then

        tmpPrimary = tmpPrimary & "K"
        tmpSecondary = tmpSecondary & "K"
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
    End If

    If ContainsAnyOf(Mid$(InputString, iCurrentPosition, 2), "CI", "CE", "CY") Then
        ' italian vs. english
        If ContainsAnyOf(Mid$(InputString, iCurrentPosition, 3), "CIO", "CIE", "CIA") Then

            tmpPrimary = tmpPrimary & "S"
            tmpSecondary = tmpSecondary & "X"
        Else

            tmpPrimary = tmpPrimary & "S"
            tmpSecondary = tmpSecondary & "S"
        End If
        
        iCurrentPosition = iCurrentPosition + 2
         GoTo GetOutOfSelect
    End If
    
    tmpPrimary = tmpPrimary & "K"
    tmpSecondary = tmpSecondary & "K"
        
    ' name sent in 'mac caffrey', 'mac gregor
    If ContainsAnyOf(Mid$(InputString, iCurrentPosition + 1, 2), " C", " Q", " G") Then
        iCurrentPosition = iCurrentPosition + 3
    Else
        If InStr(1, "CKQ", Mid$(InputString, iCurrentPosition + 1, 1)) _
        And (Mid$(InputString, iCurrentPosition + 1, 2) = "CE" _
            Or Mid$(InputString, iCurrentPosition + 1, 2) = "CI") Then
            iCurrentPosition = iCurrentPosition + 2
        Else
            iCurrentPosition = iCurrentPosition + 1
        End If
    End If
    
Case "D"
    If Mid$(InputString, iCurrentPosition, 2) = "DG" Then
        If InStr(1, "EIY", Mid$(InputString, iCurrentPosition + 2, 1)) Then
            ' e.g. 'edge'

            tmpPrimary = tmpPrimary & "J"
            tmpSecondary = tmpSecondary & "J"
            iCurrentPosition = iCurrentPosition + 3
            GoTo GetOutOfSelect
        Else
            ' e.g. 'edgar'

            tmpPrimary = tmpPrimary & "TK"
            tmpSecondary = tmpSecondary & "TK"
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
        End If
    End If
    
    If Mid$(InputString, iCurrentPosition, 2) = "DT" _
    Or Mid$(InputString, iCurrentPosition, 2) = "DD" Then
        ' MetaphAdd ("T")
        tmpPrimary = tmpPrimary & "T"
        tmpSecondary = tmpSecondary & "T"
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
    End If
        
    tmpPrimary = tmpPrimary & "T"
    tmpSecondary = tmpSecondary & "T"
    iCurrentPosition = iCurrentPosition + 1

Case "F"
        
        If Mid$(InputString, iCurrentPosition + 1, 1) = "F" Then
                iCurrentPosition = iCurrentPosition + 2
        Else
                iCurrentPosition = iCurrentPosition + 1
        End If
        
        tmpPrimary = tmpPrimary & "F"
        tmpSecondary = tmpSecondary & "F"

Case "G"
    If Mid$(InputString, iCurrentPosition + 1, 1) = "H" Then
        
        If iCurrentPosition > 1 And InStr(1, "AEIOUY", MyMid$(InputString, iCurrentPosition - 1, 1)) < 1 Then
          
                tmpPrimary = tmpPrimary & "K"
                tmpSecondary = tmpSecondary & "K"
                iCurrentPosition = iCurrentPosition + 2
                GoTo GetOutOfSelect
        End If
            
        If iCurrentPosition < 4 Then
          
            ' 'ghislane', ghiradelli
            If iCurrentPosition = 1 Then
            
                If Mid$(InputString, iCurrentPosition + 2, 1) = "I" Then

                    tmpPrimary = tmpPrimary & "J"
                    tmpSecondary = tmpSecondary & "J"
                Else

                    tmpPrimary = tmpPrimary & "K"
                    tmpSecondary = tmpSecondary & "K"
                    iCurrentPosition = iCurrentPosition + 2
                    GoTo GetOutOfSelect
                End If
            End If
        End If

          ' Parker's rule (with some further refinements) - e.g., 'hugh'
          If (iCurrentPosition > 2 And ContainsAnyOf(MyMid$(InputString, (iCurrentPosition - 2), 1), "B", "H", "D") _
            Or (iCurrentPosition > 3 And ContainsAnyOf(MyMid$(InputString, (iCurrentPosition - 3), 1), "B", "H", "D")) _
            Or (iCurrentPosition > 4 And ContainsAnyOf(MyMid$(InputString, (iCurrentPosition - 4), 1), "B", "H"))) Then
          
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
            
          Else
            ' e.g., 'laugh', 'McLaughlin', 'cough', 'gough', 'rough', 'tough'
            If (iCurrentPosition > 3 _
              And MyMid$(InputString, iCurrentPosition - 1, 1) = "U" _
              And ContainsAnyOf(MyMid$(InputString, (iCurrentPosition - 3), 1), "C", "G", "L", "R", "T")) Then
            

                tmpPrimary = tmpPrimary & "F"
                tmpSecondary = tmpSecondary & "F"
            
            Else
              If (iCurrentPosition > 1 And MyMid$(InputString, iCurrentPosition - 1, 1) <> "I") Then

                tmpPrimary = tmpPrimary & "K"
                tmpSecondary = tmpSecondary & "K"
            End If
        End If
            
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
            End If
        'End If
    End If
        
    If MyMid$(InputString, iCurrentPosition + 1, 1) = "N" Then
        
        If iCurrentPosition = 2 And InStr(1, "AEIOUY", Left$(InputString, 1)) And (IsSlavoGermanic(InputString) = False) Then
          
            tmpPrimary = tmpPrimary & "KN"
            tmpSecondary = tmpSecondary & "N"
        Else
            
            ' not e.g. 'cagney'
            If (MyMid$(InputString, (iCurrentPosition + 2), 2) <> "EY") _
            And MyMid$(InputString, iCurrentPosition + 1, 1) <> "Y" And Not IsSlavoGermanic(InputString) Then
            
                tmpPrimary = tmpPrimary & "N"
                tmpSecondary = tmpSecondary & "KN"
            Else

                tmpPrimary = tmpPrimary & "KN"
                tmpSecondary = tmpSecondary & "KN"
            End If
        End If
                
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
        
    End If
                
    ' 'tagliaro'
    If MyMid$(InputString, (iCurrentPosition + 1), 2) = "LI" And IsSlavoGermanic(InputString) = False Then
        
        tmpPrimary = tmpPrimary & "KL"
        tmpSecondary = tmpSecondary & "L"
        iCurrentPosition = iCurrentPosition + 2
    GoTo GetOutOfSelect
            
    End If
        

    ' -ges-,-gep-,-gel-, -gie- at beginning
    If (iCurrentPosition = 1 _
    And (MyMid$(InputString, iCurrentPosition + 1, 1) = "Y" _
    Or ContainsAnyOf(Mid$(InputString, (iCurrentPosition + 1), 2), "ES", "EP", "EB", "EL", "EY", "IB", "IL", "IN", "IE", "EI", "ER"))) Then
        
        tmpPrimary = tmpPrimary & "K"
        tmpSecondary = tmpSecondary & "J"
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
            
    End If
        
   
    
    '  -ger-,  -gy-
    If ((MyMid$(InputString, (iCurrentPosition + 1), 2) = "ER") Or (MyMid$(InputString, iCurrentPosition + 1, 1) = "Y")) _
    And (Not ContainsAnyOf(MyMid$(InputString, 1, 6), "DANGER", "RANGER", "MANGER")) _
    And (InStr(1, "EI", MyMid$(InputString, iCurrentPosition - 1, 1)) < 1) _
    And (Not ContainsAnyOf(MyMid$(InputString, (iCurrentPosition - 1), 3), "RGY", "OGY")) Then
        
        tmpPrimary = tmpPrimary & "K"
        tmpSecondary = tmpSecondary & "J"
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
            
    End If

    '  italian e.g, 'biaggi'
    If InStr(1, "EIY", (MyMid$(InputString, (iCurrentPosition + 1), 1))) _
    Or ContainsAnyOf(MyMid$(InputString, (iCurrentPosition - 1), 4), "AGGI", "OGGI") Then
        
        ' obvious germanic
        If ((ContainsAnyOf(MyMid$(InputString, 1, 4), "VAN ", "VON ") Or MyMid$(InputString, 1, 3) = "SCH") _
                Or MyMid$(InputString, (iCurrentPosition + 1), 2) = "ET") Then

                tmpPrimary = tmpPrimary & "K"
                tmpSecondary = tmpSecondary & "K"
            
        End If
          
            ' always soft if french ending
        If MyMid$(InputString, (iCurrentPosition + 1), 4) = "IER " Then

                tmpPrimary = tmpPrimary & "J"
                tmpSecondary = tmpSecondary & "J"
            Else

                tmpPrimary = tmpPrimary & "J"
                tmpSecondary = tmpSecondary & "K"
            End If
        
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
        
        End If

        
        If MyMid$(InputString, iCurrentPosition + 1, 1) = "G" Then
            
            tmpPrimary = tmpPrimary & "K"
            tmpSecondary = tmpSecondary & "K"
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
        Else
            
            tmpPrimary = tmpPrimary & "K"
            tmpSecondary = tmpSecondary & "K"
            iCurrentPosition = iCurrentPosition + 1
            GoTo GetOutOfSelect
        End If

Case "H"
        
    ' only keep if first & before vowel or btw. 2 vowels
    If iCurrentPosition = 1 Or InStr(1, "AEIOUY", MyMid$(InputString, iCurrentPosition - 1, 1)) Then
        If InStr(1, "AEIOUY", Mid$(InputString, iCurrentPosition + 1, 1)) Then

            tmpPrimary = tmpPrimary & "H"
            tmpSecondary = tmpSecondary & "H"
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
        End If
    End If ' also takes care of 'HH'
    
    iCurrentPosition = iCurrentPosition + 1

Case "J"

    ' obvious spanish, 'jose', 'san jacinto'
    If Mid$(InputString, iCurrentPosition, 4) = "JOSE" Or Left$(InputString, 4) = "SAN " Then
    
        If ((iCurrentPosition = 1) And ((Mid$(InputString, iCurrentPosition + 4, 1) = " ") Or (Left$(InputString, 4) = "SAN "))) Then

            tmpPrimary = tmpPrimary & "H"
            tmpSecondary = tmpSecondary & "H"

        Else
            tmpPrimary = tmpPrimary & "J"
            tmpSecondary = tmpSecondary & "H"
        End If

        iCurrentPosition = iCurrentPosition + 1
        GoTo GetOutOfSelect

    End If

    If (iCurrentPosition = 1) And (Mid$(InputString, iCurrentPosition, 4) <> "JOSE") Then

        tmpPrimary = tmpPrimary & "J"
        tmpSecondary = tmpSecondary & "A"

    Else

        ' spanish pron. of e.g. 'bajador'
        If InStr(1, "AEIOUY", MyMid$(InputString, iCurrentPosition - 1, 1)) > 0 _
            And (IsSlavoGermanic(InputString) = False) _
            And InStr(1, "AO", Mid$(InputString, iCurrentPosition + 1, 1)) Then

                    tmpPrimary = tmpPrimary & "J"
                    tmpSecondary = tmpSecondary & "H"
                    
        Else

            If iCurrentPosition = Last Then

            tmpPrimary = tmpPrimary & "J"

            Else
                If (InStr(1, "LTKSNMBZ", Mid$(InputString, iCurrentPosition + 1, 1)) < 1) _
                And (InStr(1, "SKL", MyMid$(InputString, iCurrentPosition - 1, 1)) < 1) Then

                    tmpPrimary = tmpPrimary & "J"
                    tmpSecondary = tmpSecondary & "J"

                End If
            End If
        End If
    End If

    If Mid$(InputString, iCurrentPosition + 1, 1) = "J" Then
        iCurrentPosition = iCurrentPosition + 2

    Else

        iCurrentPosition = iCurrentPosition + 1

    End If


Case "K"
        
    If Mid$(InputString, iCurrentPosition + 1, 1) = "K" Then
        iCurrentPosition = iCurrentPosition + 2
    Else
        iCurrentPosition = iCurrentPosition + 1
    End If

    tmpPrimary = tmpPrimary & "K"
    tmpSecondary = tmpSecondary & "K"
    
Case "L"

If Mid$(InputString, iCurrentPosition + 1, 1) = "L" Then

    ' spanish e.g. 'cabrillo', 'gallegos'
    If (((iCurrentPosition = Last - 2) _
    And ContainsAnyOf(MyMid$(InputString, iCurrentPosition - 1, 4), "ILLO", "ILLA", "ALLE")) _
    Or ((ContainsAnyOf(MyMid$(InputString, Last - 1, 2), "AS", "OS") Or InStr(1, "AO", Mid$(InputString, Last, 1))) _
    And MyMid$(InputString, iCurrentPosition - 1, 4) = "ALLE")) Then
    
        tmpPrimary = tmpPrimary & "L"
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
    End If

iCurrentPosition = iCurrentPosition + 2

Else
    iCurrentPosition = iCurrentPosition + 1
End If

tmpPrimary = tmpPrimary & "L"
tmpSecondary = tmpSecondary & "L"

    
Case "M"
        If (MyMid$(InputString, iCurrentPosition - 1, 3) = "UMB" _
            And (iCurrentPosition + 1 = Last _
            Or Mid$(InputString, iCurrentPosition + 2, 2) = "ER")) _
        Or Mid$(InputString, iCurrentPosition + 1, 1) = "M" Then
        
                iCurrentPosition = iCurrentPosition + 2
        Else
                iCurrentPosition = iCurrentPosition + 1
        End If

        tmpPrimary = tmpPrimary & "M"
        tmpSecondary = tmpSecondary & "M"

Case "N"
        
        If Mid$(InputString, iCurrentPosition + 1, 1) = "N" Then
            iCurrentPosition = iCurrentPosition + 2
        Else
            iCurrentPosition = iCurrentPosition + 1
        End If

        tmpPrimary = tmpPrimary & "N"
        tmpSecondary = tmpSecondary & "N"

Case ""
        
        tmpPrimary = tmpPrimary & "N"
        tmpSecondary = tmpSecondary & "N"
        iCurrentPosition = iCurrentPosition + 1
        GoTo GetOutOfSelect

Case "P"
        
    If Mid$(InputString, iCurrentPosition + 1, 1) = "H" Then

        tmpPrimary = tmpPrimary & "F"
        tmpSecondary = tmpSecondary & "F"
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
    End If
        
    ' Also account for CAMPBELL and RASPBERRY
    If InStr(1, "PB", Mid$(InputString, iCurrentPosition + 1, 1)) Then
        iCurrentPosition = iCurrentPosition + 2
    Else
        iCurrentPosition = iCurrentPosition + 1
    End If

    tmpPrimary = tmpPrimary & "P"
    tmpSecondary = tmpSecondary & "P"

Case "Q"

        If Mid$(InputString, iCurrentPosition + 1, 1) = "Q" Then
            iCurrentPosition = iCurrentPosition + 2
        Else
            iCurrentPosition = iCurrentPosition + 1
        End If

        tmpPrimary = tmpPrimary & "K"
        tmpSecondary = tmpSecondary & "K"

Case "R"
        ' french e.g. 'rogier', but exclude 'hochmeier'
        If iCurrentPosition = Last _
        And IsSlavoGermanic(InputString) = False _
        And MyMid$(InputString, iCurrentPosition - 2, 2) = "IE" _
        And MyMid$(InputString, iCurrentPosition - 4, 2) <> "ME" _
            And MyMid$(InputString, iCurrentPosition - 4, 2) <> "MA" Then

                tmpSecondary = tmpSecondary & "R"
        Else

                tmpPrimary = tmpPrimary & "R"
                tmpSecondary = tmpSecondary & "R"
        End If
        
        If Mid$(InputString, iCurrentPosition + 1, 1) = "R" Then
            iCurrentPosition = iCurrentPosition + 2
        Else
            iCurrentPosition = iCurrentPosition + 1
        End If
        
Case "S"
        ' special cases 'island', 'isle', 'carlisle', 'carlysle'
        If MyMid$(InputString, iCurrentPosition - 1, 3) = "ISL" _
        Or MyMid$(InputString, iCurrentPosition - 1, 3) = "YSL" Then
            iCurrentPosition = iCurrentPosition + 1
            GoTo GetOutOfSelect
        End If

        ' special case 'sugar-'
        If iCurrentPosition = 1 And Left$(InputString, 5) = "SUGAR" Then
            tmpPrimary = tmpPrimary & "X"
            tmpSecondary = tmpSecondary & "S"
            iCurrentPosition = iCurrentPosition + 1
            GoTo GetOutOfSelect
        End If
        

        If Mid$(InputString, iCurrentPosition, 2) = "SH" Then
            ' Germanic
            If ContainsAnyOf(Mid$(InputString, iCurrentPosition + 1, 4), "HEIM", "HOEK", "HOLM", "HOLZ") Then
                tmpPrimary = tmpPrimary & "S"
                tmpSecondary = tmpSecondary & "S"
            Else
                ' MetaphAdd ("X")
                tmpPrimary = tmpPrimary & "X"
                tmpSecondary = tmpSecondary & "X"
            End If
            
            iCurrentPosition = iCurrentPosition + 2
            GoTo GetOutOfSelect
            
        End If

        ' Italian & Armenian
        If Mid$(InputString, iCurrentPosition, 3) = "SIO" _
        Or Mid$(InputString, iCurrentPosition, 3) = "SIA" _
        Or Mid$(InputString, iCurrentPosition, 4) = "SIAN" Then
        
            If IsSlavoGermanic(InputString) = False Then
                tmpPrimary = tmpPrimary & "S"
                tmpSecondary = tmpSecondary & "X"
            Else
                tmpPrimary = tmpPrimary & "S"
                tmpSecondary = tmpSecondary & "S"
            End If
            
            iCurrentPosition = iCurrentPosition + 3
            GoTo GetOutOfSelect
        End If

        ' german & anglicisations, e.g. 'smith' match 'schmidt', 'snider' match 'schneider'
        ' also, -sz- in slavic language altho in hungarian it is pronounced 's'
        
        If (iCurrentPosition = 1) _
        And InStr(1, "MNLW", Mid$(InputString, iCurrentPosition + 1, 1)) _
        Or Mid$(InputString, iCurrentPosition + 1, 1) = "Z" Then

            tmpPrimary = tmpPrimary & "S"
            tmpSecondary = tmpSecondary & "X"
            
            If Mid$(InputString, iCurrentPosition + 1, 1) = "Z" Then
                iCurrentPosition = iCurrentPosition + 2
            Else
                iCurrentPosition = iCurrentPosition + 1
            End If

            GoTo GetOutOfSelect
        End If

                If Mid$(InputString, iCurrentPosition, 2) = "SC" Then
                                        ' Schlesinger's rule
                    If Mid$(InputString, iCurrentPosition + 2, 1) = "H" Then
                                                ' dutch origin, e.g. 'school', 'schooner'
                        If ContainsAnyOf(Mid$(InputString, iCurrentPosition + 3, 2), "OO", "ER", "EN", "UY", "ED", "EM") Then

                                                        ' 'schermerhorn', 'schenker'
                            If Mid$(InputString, iCurrentPosition + 3, 2) = "ER" _
                            Or Mid$(InputString, iCurrentPosition + 3, 2) = "EN" Then

                                tmpPrimary = tmpPrimary & "X"
                                tmpSecondary = tmpSecondary & "SK"

                            Else

                                tmpPrimary = tmpPrimary & "SK"
                                tmpSecondary = tmpSecondary & "SK"
                            End If

                                                        iCurrentPosition = iCurrentPosition + 3
                                                        GoTo GetOutOfSelect

                        Else
                            If iCurrentPosition = 1 _
                            And (InStr(1, "AEIOUYW", Mid$(InputString, 4, 1)) < 1) Then

                                tmpPrimary = tmpPrimary & "X"
                                tmpSecondary = tmpSecondary & "S"

                                                        Else
                                tmpPrimary = tmpPrimary & "X"
                                tmpSecondary = tmpSecondary & "X"
                            End If

                                                        iCurrentPosition = iCurrentPosition + 3
                                                        GoTo GetOutOfSelect
                        End If

                    ElseIf InStr(1, "IEY", Mid$(InputString, iCurrentPosition + 2, 1)) > 0 Then

                        tmpPrimary = tmpPrimary & "S"
                        tmpSecondary = tmpSecondary & "S"
                        iCurrentPosition = iCurrentPosition + 3
                        GoTo GetOutOfSelect

                    End If

                    ' otherwise
                    tmpPrimary = tmpPrimary & "SK"
                    tmpSecondary = tmpSecondary & "SK"
                    iCurrentPosition = iCurrentPosition + 3
                    GoTo GetOutOfSelect
                    
                End If
                




        ' french e.g. 'resnais', 'artois'
        If iCurrentPosition = Last _
        And (MyMid$(InputString, iCurrentPosition - 2, 2) = "AI" _
            Or MyMid$(InputString, iCurrentPosition - 2, 2) = "OI") Then
            

            tmpSecondary = tmpSecondary & "S"
        Else

            tmpPrimary = tmpPrimary & "S"
            tmpSecondary = tmpSecondary & "S"
        End If


        If InStr(1, "SZ", Mid$(InputString, iCurrentPosition + 1, 1)) Then
            iCurrentPosition = iCurrentPosition + 2
        Else
            iCurrentPosition = iCurrentPosition + 1
        End If

    ' End If
    


Case "T"

    If Mid$(InputString, iCurrentPosition, 4) = "TION" Then

        tmpPrimary = tmpPrimary & "X"
        tmpSecondary = tmpSecondary & "X"
        iCurrentPosition = iCurrentPosition + 3
        GoTo GetOutOfSelect
    End If
    
    If Mid$(InputString, iCurrentPosition, 3) = "TIA" _
        Or Mid$(InputString, iCurrentPosition, 3) = "TCH" Then

        tmpPrimary = tmpPrimary & "X"
        tmpSecondary = tmpSecondary & "X"
        iCurrentPosition = iCurrentPosition + 3
        GoTo GetOutOfSelect
    End If

    If Mid$(InputString, iCurrentPosition, 2) = "TH" _
        Or Mid$(InputString, iCurrentPosition, 3) = "TTH" Then
        
        ' special case 'thomas', 'thames' or germanic
        
        If Mid$(InputString, iCurrentPosition + 2, 2) = "OM" _
        Or Mid$(InputString, iCurrentPosition + 2, 2) = "AM" _
        Or Left$(InputString, 4) = "VAN " _
        Or Left$(InputString, 4) = "VON " _
        Or Left$(InputString, 3) = "SCH" Then
        
            tmpPrimary = tmpPrimary & "T"
            tmpSecondary = tmpSecondary & "T"

        Else
            tmpPrimary = tmpPrimary & "0"
            tmpSecondary = tmpSecondary & "T"
        End If
        
    iCurrentPosition = iCurrentPosition + 2
    GoTo GetOutOfSelect
    
    End If

    If InStr(1, "TD", Mid$(InputString, iCurrentPosition + 1, 1)) Then
        iCurrentPosition = iCurrentPosition + 2
    Else
        iCurrentPosition = iCurrentPosition + 1
    End If
    
    tmpPrimary = tmpPrimary & "T"
    tmpSecondary = tmpSecondary & "T"
    

Case "V"

        If Mid$(InputString, iCurrentPosition + 1, 1) = "V" Then
            iCurrentPosition = iCurrentPosition + 2
        Else
            iCurrentPosition = iCurrentPosition + 1
        End If
        
        tmpPrimary = tmpPrimary & "F"
        tmpSecondary = tmpSecondary & "F"
        GoTo GetOutOfSelect

Case "W"
    ' can also be in middle of word
    If Mid$(InputString, iCurrentPosition, 2) = "WR" Then

        tmpPrimary = tmpPrimary & "R"
        tmpSecondary = tmpSecondary & "R"
        iCurrentPosition = iCurrentPosition + 2
        GoTo GetOutOfSelect
    End If

    If iCurrentPosition = 1 _
    And (InStr(1, "AEIOUY", Mid$(InputString, iCurrentPosition + 1, 1)) _
        Or Mid$(InputString, iCurrentPosition, 2) = "WH") Then
                
        ' Wasserman should match Vasserman
        If InStr(1, "AEIOUY", Mid$(InputString, iCurrentPosition + 1, 1)) Then

            tmpPrimary = tmpPrimary & "A"
            tmpSecondary = tmpSecondary & "F"
        Else
            ' need Uomo to match Womo
            tmpPrimary = tmpPrimary & "A"
            tmpSecondary = tmpSecondary & "A"
        End If
    End If

    ' Arnow should match Arnoff
    If (iCurrentPosition = Last And InStr(1, "AEIOUY", MyMid$(InputString, iCurrentPosition - 1, 1)) _
    Or ContainsAnyOf(MyMid$(InputString, iCurrentPosition - 1, 5), "EWSKI", "EWSKY", "OWSKI", "OWSKY") _
    Or Left$(InputString, 3) = "SCH") Then

        tmpSecondary = tmpSecondary & "F"
        iCurrentPosition = iCurrentPosition + 1
        GoTo GetOutOfSelect
    End If

    ' polish e.g. 'filipowicz'
    If Mid$(InputString, iCurrentPosition, 4) = "WICZ" _
    Or Mid$(InputString, iCurrentPosition, 4) = "WITZ" Then
    
        tmpPrimary = tmpPrimary & "TS"
        tmpSecondary = tmpSecondary & "FX"
        iCurrentPosition = iCurrentPosition + 4
        GoTo GetOutOfSelect
    End If

    ' else skip it
    iCurrentPosition = iCurrentPosition + 1

Case "X"
        ' french e.g. breaux
        If Not (iCurrentPosition = Last And (MyMid$(InputString, iCurrentPosition - 3, 3) = "IAU" _
            Or MyMid$(InputString, iCurrentPosition - 3, 3) = "IAU" _
            Or MyMid$(InputString, iCurrentPosition - 2, 2) = "AU" Or MyMid$(InputString, iCurrentPosition - 2, 2) = "OU")) Then
            
            tmpPrimary = tmpPrimary & "KS"
            tmpSecondary = tmpSecondary & "KS"
        End If
        
        If InStr(1, "CX", Mid$(InputString, iCurrentPosition + 1, 1)) Then
            iCurrentPosition = iCurrentPosition + 1
        End If

        iCurrentPosition = iCurrentPosition + 1

Case "Z"
        ' Chinese pinyin e.g. 'zhao'
        If Mid$(InputString, iCurrentPosition + 1, 1) = "H" Then
                tmpPrimary = tmpPrimary & "J"
                tmpSecondary = tmpSecondary & "J"
                iCurrentPosition = iCurrentPosition + 2
                GoTo GetOutOfSelect
        End If
        
        If (ContainsAnyOf(Mid$(InputString, iCurrentPosition + 1, 2), "ZO", "ZI", "ZA") _
        Or (IsSlavoGermanic(InputString) _
        And (iCurrentPosition > 1 And MyMid$(InputString, iCurrentPosition - 1, 1) <> "T"))) Then

            tmpPrimary = tmpPrimary & "S"
            tmpSecondary = tmpSecondary & "TS"
        Else
            tmpPrimary = tmpPrimary & "S"
            tmpSecondary = tmpSecondary & "S"
        End If
        

        If Mid$(InputString, iCurrentPosition + 1, 1) = "Z" Then
                iCurrentPosition = iCurrentPosition + 2
        Else
                iCurrentPosition = iCurrentPosition + 1
        End If
        
Case Else
        
        iCurrentPosition = iCurrentPosition + 1
End Select

Loop


DoubleMetaphone.Primary = tmpPrimary
DoubleMetaphone.Secondary = tmpSecondary

If tmpPrimary <> tmpSecondary Then
    DoubleMetaphone.HasAlternate = True
End If

End Function


